function [sig] = fct_dB_to_pressure(sig_dB,spectrum,fs,ref80dB)
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% E. Ponsot, May 2012                                                   
% 
% convert dB profile to SPL
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%

pref=ref80dB/(10^(80/20)); % put 0.283 as reference for use of Genesis Loudness ToolBox


duration=length(sig_dB)/fs;
nbpoints=length(sig_dB);
timevect=linspace(0,duration,nbpoints);

profile_rms=zeros(length(sig_dB),1);

for i=1:length(sig_dB)
profile_rms(i)=pref*10^(sig_dB(i)/20);
end



if strcmp(spectrum, 'wn')==1

x = rand(1,nbpoints);  % nbpoints white noise samples, 
                  %   uniform between 0 and 1.
xn = 2*(x-0.5);   % Make it uniform between -1 and +1

signal=xn';

elseif strcmp(spectrum, 'wnr')==1

    time_window=0.1;    
    nb_points_window=time_window*fs;

    x = rand(1,nb_points_window);  % nbpoints white noise samples, 
                      %   uniform between 0 and 1.
    xn = 2*(x-0.5);   % Make it uniform between -1 and +1

    y=[];

    for ii = 1 : floor(nbpoints/nb_points_window)
     y=[y  xn];
    end

    if (nbpoints-nb_points_window*floor(nbpoints/nb_points_window)) > 0
    for jj = 1 : nbpoints-nb_points_window*floor(nbpoints/nb_points_window)
        y=[y  xn(jj)];
    end
    end
    signal=y';
    
    
elseif strcmp(spectrum, 'narrow_100')==1 ||  strcmp(spectrum, 'narrow_400')==1
  
%Analytical expressions for critical?band rate and critical bandwidth as a function of frequency 
% Zwicker & Terhart, JASA, 1980

% at 1000 Hz, the critical ban width is 160 Hz

% Choose the lower and upper freq bounds
    if strcmp(spectrum, 'narrow_100')==1
    % set variables for filter
    lf = 950;   % lowest frequency
    hf = 1050;   % highest frequency
    elseif strcmp(spectrum, 'narrow_400')==1
    lf = 800;   % lowest frequency
    hf = 1200;   % highest frequency   
    end

lp = lf * duration; % ls point in frequency domain    
hp = hf * duration; % hf point in frequency domain

% design filter
n = fs * duration;  % number of samples
filter = zeros(1, n);           % initializaiton by 0
filter(1, lp : hp) = 1;         % filter design in real number
filter(1, n - hp : n - lp) = 1; % filter design in imaginary number



x = rand(1,nbpoints);  % nbpoints white noise samples, 
                  %   uniform between 0 and 1.
xn = 2*(x-0.5);   % Make it uniform between -1 and +1
  
%signal=xn'; 

% do filter
s = fft(xn);                  % FFT
s = s .* filter;                 % filtering
s = ifft(s);                     % inverse FFT
s = real(s);
signal=s'./max(abs(s));


elseif strcmp(spectrum, 'narrow_100_but')==1 ||  strcmp(spectrum, 'narrow_400_but')==1
  
%Analytical expressions for critical?band rate and critical bandwidth as a function of frequency 
% Zwicker & Terhart, JASA, 1980

% at 1000 Hz, the critical ban width is 160 Hz
Fs=fs;
RP=2;
RS=40;

ButterOrder=8; % Order for the Butterworth filters

y=2*rand(1,duration*Fs)-1;

% Choose the lower and upper freq bounds
    if strcmp(spectrum, 'narrow_100_but')==1
FH1=1050;
FH2=1060;
FL1=940;
FL2=950;
    elseif strcmp(spectrum, 'narrow_400_but')==1
FH1=1200;
FH2=1210;
FL1=790;
FL2=800; 
    end

%% Elliptic filters
%y=lowpass(y,FH1,FH2,RP,RS,Fs);
% lowpass
Wp=FH1*2/Fs;
Ws=FH2*2/Fs;
[N Wn]=ellipord(Wp,Ws,RP,RS);
[B A]=ellip(N,RP,RS,Wn,'low');
y=filter(B,A,y); %#ok<NODEF>

% highpass
Wp=FL2*2/Fs;
Ws=FL1*2/Fs;
[N Wn]=ellipord(Wp,Ws,RP,RS);
[B A]=ellip(N,RP,RS,Wn,'high');
y=filter(B,A,y);
y=.95*y/max(abs(y));

% Add butter filter
[B,A] = butter(ButterOrder,2*FH1/Fs,'low');
y=filter(B,A,y);
%y=filter(B,A,y);
%y=filter(B,A,y);
[B,A] = butter(ButterOrder,2*FL2/Fs,'high');
y=filter(B,A,y);
%y=filter(B,A,y);
%y=filter(B,A,y);

y=y/max(abs(y));
signal=y';



elseif strcmp(spectrum, '400r')==1

    time_window=0.8;    
    nb_points_window=time_window*fs; 
    t=0:1/fs:time_window-1/fs;
    percent_change=50;

    y=[];
    f0=400*unifrnd((100-percent_change)/100,(100+percent_change)/100);
    %f0 = 400 + 400*(percent_change/100)*randn(1);
    signal=sin(2*pi*f0*t(:));
    y=[y ; signal];
     
    for ii = 1 : floor(nbpoints/nb_points_window)-1
     
     f0=400*unifrnd((100-percent_change)/100,(100+percent_change)/100);
     %f0 = 400 + 400*(percent_change/100)*randn(1);
     der=(y(end)-y(end-1))*fs;
     diff=1;
     t=0:1/fs:time_window-1/fs;
     pp=0;
     sign_test=-sign(der);
     while (diff > .005 || sign_test ~= sign(der) )
         
         
         signal=sin(2*pi*f0*t(:));
         diff=abs(signal(1)-y(end));
         sign_test=sign(signal(2)-signal(1));
         
         pp=pp+1/fs;
      
         t=pp:1/fs:time_window-1/fs+pp;
     end
     
     y=[y ; signal];
    
    end
    
    
    
    if (nbpoints-nb_points_window*floor(nbpoints/nb_points_window)) > 0
    f0=400*unifrnd((100-percent_change)/100,(100+percent_change)/100);
     %f0 = 400 + 400*(percent_change/100)*randn(1);
     der=(y(end)-y(end-1))*fs;
     diff=1;
     t=0:1/fs:time_window-1/fs;
     pp=0;
     sign_test=-sign(der);
     signal=sin(2*pi*f0*t(:));
     
     while (diff > .005 || sign_test ~= sign(der) )
         
         
         signal=sin(2*pi*f0*t(:));
         diff=abs(signal(1)-y(end));
         sign_test=sign(signal(2)-signal(1));
         
         pp=pp+1/fs;
      
         t=pp:1/fs:time_window-1/fs+pp;
     end
     
     
    for jj = 1 : nbpoints-nb_points_window*floor(nbpoints/nb_points_window)
        y=[y ; signal(jj)];
    end
    end

    
    signal=y;  

elseif strcmp(spectrum, 'triangle400')==1
    
    f0=400;
    t=0:1/fs:duration-1/fs;
    y=sawtooth(2*pi*f0*t,0.5);
    signal=y';

elseif strcmp(spectrum, 'ramp_400-500')==1
    
    f0=400;
    t=0:1/fs:duration-1/fs;
    percent_change=-10;
    randfreq_v=1:percent_change/(100*(nbpoints-1)):1+percent_change/100;

    y=sin(2*pi*f0*randfreq_v.*t);
    
    signal=y';

elseif strcmp(spectrum, 'sin_400-500')==1
    
    f0=400;
    f1=5000;
    t=0:1/fs:duration-1/fs;
    percent_change=2;
    f_var=percent_change/100*f0*sin(2*pi*f1*t);
    
    y=sin(2*pi*(f0+f_var).*t);
    
    signal=y';
    
elseif strcmp(spectrum, 'tr_400-500')==1
    
    f0=400;
    f1=2;
    t=0:1/fs:duration-1/fs;
    percent_change=1;
    f_var=percent_change/100*f0*sin(2*pi*f1*t);
    
    y=sawtooth(2*pi*(f0+f_var).*t,0.5);
    
    signal=y';    

else

f0=spectrum;  
t=0:1/fs:duration-1/fs;
y=sin(2*pi*f0*t);
signal=y';

end


coef = sqrt(1/(nbpoints)*sum((signal(:)).^2));  

for i=1:length(sig_dB)  
profile_rms(i)=pref*10^(sig_dB(i)/20)./coef;
end
sig=signal.*profile_rms;


end
